//+------------------------------------------------------------------+
//|                                            SuperJolt Snapper.mq4 |
//|                                        Copyright 2015, SuperJolt |
//|                                             http://SuperJolt.com |
//+------------------------------------------------------------------+

#property copyright "Copyright 2015, SuperJolt"
#property link      "http://SuperJolt.com"
#property version   "1.00"
#property strict
#property indicator_chart_window

//+-----------------------------------------------------------------+
#import "SuperJolt.MT4.Snapper.dll"
   int _CreateIndicatorInstance(int hostFrameHandle, string symbolName, int period, string dataFilesPath, string screenshotsFilesPath, int corner);
   void _TakeScreenshot(int hostFrameHandle);
   void _RemoveIndicatorInstance(int hostFrameHandle);
#import

#import "user32.dll"
int SendMessageW(int hWnd,int Msg,int wParam,int lParam);
int GetParent(int hWnd);
#import

#define WM_MDIACTIVATE  0x222
//+------------------------------------------------------------------+

int hostFrameHandle;

string OLabel = "Snapper";

extern string screenshotsFilePath = ""; //Path to save screenshots, blank to use default
extern int corner=1; //Corner: 0=top left, 1=bottom left
extern bool orderScreenshots = true; //Create screenshots on order open/close

//+------------------------------------------------------------------+

bool isIndicatorInitialised = false;
int orderCount = 0;
//+------------------------------------------------------------------+

int OnInit()
{
   if(!IsDllsAllowed()) {
      Alert("Error: DLL calls must be allowed for Snapper to open. Please check your settings");
      return(INIT_FAILED);
   }
   
   orderCount = OrdersTotal();
   hostFrameHandle = WindowHandle(Symbol(), Period());
  
   
   string dataPath=TerminalInfoString(TERMINAL_DATA_PATH);
   int result = _CreateIndicatorInstance(hostFrameHandle, Symbol(), Period(), dataPath, screenshotsFilePath, corner);
   
   if (result<0)
   {
      Print("Error - CreateInstance failed");
      return(INIT_FAILED);
   }
   
   if (orderScreenshots)
      EventSetTimer(1);
   
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
void OnTimer()
{
   if(orderCount==OrdersTotal()) return;
   
   //Close Trade?
   if(OrdersTotal()<orderCount)
   {
      if(OrderSelect(OrdersHistoryTotal()-1,SELECT_BY_POS,MODE_HISTORY))
      {
         if(OrderSymbol()==Symbol())
         {
            DrawTrade();
            BringChartToFront();
            _TakeScreenshot(hostFrameHandle);
         }
      }
   }

   //Open Trade?
   if(OrdersTotal()>orderCount)
   {
      if(OrderSelect(OrdersTotal()-1,SELECT_BY_POS,MODE_TRADES))
      {
         if(OrderSymbol()==Symbol())
         {
            BringChartToFront();
            _TakeScreenshot(hostFrameHandle);
         }
      }
   }
   
   orderCount = OrdersTotal();
}
//+------------------------------------------------------------------+

void DrawTrade()
{

   string orderType;  
   color arrowOpenColor; 
   color arrowCloseColor;     
   color orderLineColor;
 
   if(OrderType()==OP_BUY )
   { 
      orderType=" buy ";  
      arrowOpenColor = Lime; 
      arrowCloseColor = Blue;     
      orderLineColor=Blue;
   }
   else if(OrderType()==OP_SELL)
   {        
      orderType=" sell ";       
      arrowOpenColor = Red;       
      arrowCloseColor = Goldenrod;      
      orderLineColor=Red;
   }
                  
   string objectName=StringConcatenate("#",OrderTicket(),orderType,DoubleToStr(OrderLots(),2)," ",OrderSymbol()," at ",DoubleToStr(OrderOpenPrice(),Digits));
   if(ObjectFind(objectName)==-1)
   {
      ObjectCreate(objectName,OBJ_ARROW,0,OrderOpenTime(),OrderOpenPrice());
      ObjectSet(objectName, OBJPROP_ARROWCODE,1);
      ObjectSet(objectName, OBJPROP_COLOR,arrowOpenColor);
      ObjectSet(objectName, OBJPROP_SELECTABLE, False);
   }

   objectName=StringConcatenate("#",OrderTicket(),orderType,DoubleToStr(OrderLots(),2)," ",OrderSymbol()," at ",DoubleToStr(OrderOpenPrice(),Digits)," close at ",DoubleToStr(OrderClosePrice(),Digits));
   if(ObjectFind(objectName)==-1)
   {
      ObjectCreate(objectName,OBJ_ARROW,0,OrderCloseTime(),OrderClosePrice());
      ObjectSet(objectName, OBJPROP_ARROWCODE,3);
      ObjectSet(objectName, OBJPROP_COLOR,arrowCloseColor);
      ObjectSet(objectName, OBJPROP_SELECTABLE, False);
   }

   objectName=StringConcatenate("#",OrderTicket()," ",DoubleToStr(OrderOpenPrice(),Digits)," -> ",DoubleToStr(OrderClosePrice(),Digits));
   if(ObjectFind(objectName)==-1)
   {
      ObjectCreate(objectName,OBJ_TREND,0,OrderOpenTime(),OrderOpenPrice(),OrderCloseTime(),OrderClosePrice());
      ObjectSet(objectName, OBJPROP_STYLE,STYLE_DOT);
      ObjectSet(objectName, OBJPROP_COLOR,orderLineColor);
      ObjectSet(objectName, OBJPROP_RAY, False);
      ObjectSet(objectName, OBJPROP_SELECTABLE, False);
      ObjectSet(objectName,OBJPROP_RAY_RIGHT,False);
   }

}
//+------------------------------------------------------------------+
void BringChartToFront()
{
   int handle=GetParent(hostFrameHandle);

   int WinParent=GetParent(handle);
   SendMessageW(WinParent,WM_MDIACTIVATE,handle,0);
}
//+------------------------------------------------------------------+
int start()
{
   return (0);
}
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   int obj_total=ObjectsTotal();
   for(int i = obj_total - 1; i >= 0; i--)
   {
      string label = ObjectName(i);
      if(StringFind(label, OLabel)==-1) continue;
      ObjectDelete(label); 
   }     
   
   if(IsDllsAllowed())
      _RemoveIndicatorInstance(hostFrameHandle);
   
   return;
}

//+------------------------------------------------------------------+
